<?php

/**
  * SquirrelMail Change Passwd Plugin
  * Copyright (C) 2003-2006 Paul Lesneiwski <paul@squirrelmail.org>
  * Copyright (C) 2002 Thiago Melo de Paula <thiagomp@coc.com.br>
  * This program is licensed under the GNU GPL. See COPYING for details.
  *
  * @package plugins
  * @subpackage Change Passwd
  *
  */


if (file_exists('../../include/init.php'))
   include_once('../../include/init.php');
else if (file_exists('../../include/validate.php'))
{
   define('SM_PATH', '../../');
   include_once(SM_PATH . 'include/validate.php');
}
else
{
   chdir('..');
   define('SM_PATH', '../');
   include_once(SM_PATH . 'src/validate.php');
}


include_once(SM_PATH . 'functions/display_messages.php');
load_config('change_passwd', 'config.php');



global $disconn, $confirmNewPass, $confirmOldPass, $pathToPwPolicy,
       $seeOutput, $overridePathToChpasswd, $pathToPw, $debug, $minimumPasswordLength,
       $gochange, $user, $old_pw, $new_pw1, $new_pw2, $base_uri, $onetimepad, $key, 
       $username, $invalid_password_characters, $enforce_cracklib, $cracklib_dict_path;
if (empty($base_uri))
   $base_uri = sqm_baseuri();



sqgetGlobalVar('gochange', $gochange, SQ_FORM);
sqgetGlobalVar('old_pw', $old_pw, SQ_POST);
sqgetGlobalVar('new_pw1', $new_pw1, SQ_POST);
sqgetGlobalVar('new_pw2', $new_pw2, SQ_POST);
//sqgetGlobalVar('user', $user, SQ_FORM);
//$user = trim($user);
$user = $username;  // don't need to cycle this thru the form, only makes it less secure
$gochange = trim($gochange);
$old_pw = trim($old_pw);
$new_pw1 = trim($new_pw1);
$new_pw2 = trim($new_pw2);




bindtextdomain('change_passwd', SM_PATH . 'locale');
textdomain('change_passwd');


if (isset($gochange) && $gochange) 
{

   //
   // verify input
   //



   // filter out any unwanted characters
   //
   foreach ($invalid_password_characters as $char)
      if (strpos($new_pw1, $char) !== FALSE)
      {
         $msg = _("Invalid character in password:") . ' ' . $char;
         $gochange = 0;
      }


   // check for empty values
   //
   if (empty($user))
   {
      $msg = _("Could not determine your username.") . ' ' 
           . _("Please contact your system administrator.");
      $gochange = 0;
   }
   else if ($confirmOldPass && empty($old_pw))
   {
      $msg = _("Please enter your current password");
      $gochange = 0;
   }
   else if (empty($new_pw1))
   {
      $msg = _("Please enter your new password");
      $gochange = 0;
   }
   else if ($confirmNewPass && empty($new_pw2))
   {
      $msg = _("Please re-enter your new password");
      $gochange = 0;
   }


   // check for minimum password length
   //
   else if ($minimumPasswordLength > 0 && strlen($new_pw1) < $minimumPasswordLength)
   {
      $msg = sprintf(_("Your new password must be at least %s characters long"), 
                     $minimumPasswordLength);
      $gochange = 0;
   }


   // pass through crack-lib if supported...
   //
   else if ($enforce_cracklib && function_exists('crack_check')) 
   {

      // don't bother opening/closing the dictionary if not specified
      //
      if (empty($cracklib_dict_path)) 
      {
         if (crack_check($new_pw1) === FALSE) 
         {
            $msg = crack_getlastmessage();
            //$msg = _(crack_getlastmessage());
            $gochange = 0;
         }

      } 
      else 
      {
         $dictionary = crack_opendict($cracklib_dict_path);
         if (crack_check($dictionary, $new_pw1) === FALSE) 
         {
            $msg = crack_getlastmessage();
            //$msg = _(crack_getlastmessage());
            $gochange = 0;
         }
         crack_closedict($dictionary);
      }

   }


   // make sure new passwords are the same
   //
   else if ($confirmNewPass) 
   {
      if ($new_pw1 != $new_pw2)  
      {
         $msg = _("The confirmation of your new password did not match");
         $gochange = 0;
      }
   }



   // if everything else was OK, make sure old (current) 
   // password is correct
   //
   if ($gochange)
   {
      sqgetGlobalVar('key', $key, SQ_COOKIE);
      sqgetGlobalVar('onetimepad', $onetimepad, SQ_SESSION);

      $currentPassword = OneTimePadDecrypt($key, $onetimepad);

      if ($confirmOldPass && $currentPassword != $old_pw)
      {
         $msg = _("Your current password is not correct");
         // debugging... 
         // $msg .= "<br />current pwd: $currentPassword<br />old pwd: $old_pw<br />confirm old pwd: $confirmOldPass<br />";
         $gochange = 0;
      }
   }



   // sanitize input for use in exec()
   //
   $safe_newpw = escapeshellarg($new_pw1);
   $safe_oldpw = escapeshellarg($old_pw);
   $safe_user = escapeshellarg($user);
//sm_print_r($safe_newpw);exit;



   // if we got this far, we can proceed with the password update
   //
   if ($gochange)
   {


      // use pw utility (FreeBSD)
      //
      if (!empty($pathToPw)) 
      {
         $fd = popen("$pathToPw usermod $safe_user -h 0 2>&1", 'w+');
         fwrite($fd, $new_pw1 . "\n");
         $ret = fread($fd, 4096);
         pclose($fd);                
//TODO: what are the good and bad return values from pw??  For now, this hack:
         $ret = '';
      }


      // otherwise, use chpasswd
      //
      else
      {

         // use pwpolicy utility (OS X)
         //
         if (!empty($pathToPwPolicy)) 
         {
            $cmd = "$pathToPwPolicy -a $safe_user -p $safe_oldpw -u $safe_user -setpassword $safe_newpw 2>&1";
         }


         // use chpasswd in plugin directory
         //
         else if (empty($overridePathToChpasswd))
         {
            $cmd = SM_PATH . "plugins/change_passwd/chpasswd $safe_user $safe_oldpw $safe_newpw 2>&1";
         }

   
         // use chpasswd elsewhere
         //
         else
            $cmd = "$overridePathToChpasswd $safe_user $safe_oldpw $safe_newpw";


         // --------------------------------------------------------------------------------
         // print out the command for debugging
         //
         if ($debug)
         {

/* NOTE: this is useless since we can't get here unless the config file was loaded 
            if (!file_exists(SM_PATH . 'plugins/change_passwd/config.php'))
               echo '<br /><hr /><br />Missing config file!<br />Please read INSTALL file to learn how to install this plugin correctly.<br />';
*/


            // get permissions info on executable
            //
            if (!empty($pathToPwPolicy)) 
               $permissions = fileperms($pathToPwPolicy);
            else if (empty($overridePathToChpasswd))
               $permissions = fileperms(SM_PATH . 'plugins/change_passwd/chpasswd');
            else
               $permissions = fileperms($overridePathToChpasswd);

            echo '<br /><hr /><br />Permissions of chpasswd executable are: ' 
                 . base_convert($permissions, 10, 8) . '<br />';


            // is the file accessible to web server and suid?
            //
            if (!($permissions & 04000))
               echo "<br />It appears that the chpasswd file is not suid!<br />Please read the INSTALL and README files to see how to set its permissions correctly.<br />";
            if (!($permissions & 00100))
               echo "<br />It appears that the chpasswd file is not executable by its owner!<br />Please read the INSTALL and README files to see how to set its permissions correctly.<br />";
            if (!($permissions & 00010))
               echo "<br />It appears that the chpasswd file is not executable by its group!<br />Please read the INSTALL and README files to see how to set its permissions correctly.<br />";


            // if possible, check who owns the file
            //
            if (function_exists('posix_getgid'))
            {

               // ---------------------
               // check group ownership
               //
               $webGroupID = posix_getgid();
               if (!empty($pathToPwPolicy)) 
                  $groupOwnerID = filegroup($pathToPwPolicy);
               else if (empty($overridePathToChpasswd))
                  $groupOwnerID = filegroup(SM_PATH . 'plugins/change_passwd/chpasswd');
               else
                  $groupOwnerID = filegroup($overridePathToChpasswd);


               $continueCheckingGroup = TRUE;


               if ($webGroupID === FALSE)
               {
                  echo '<br /><hr /><br />Could not determine group under which the web server is running.<br />Please check README file to verify that the group of the web server and the chpasswd utility are the same.<br />';
                  $continueCheckingGroup = FALSE;
               }
               else
               {
                  $webGroup = posix_getgrgid($webGroupID);
                  if (!$webGroup)
                  {
                     echo '<br /><hr /><br />Could not determine group under which the web server is running.<br />Please check README file to verify that the group of the web server and the chpasswd utility are the same.<br />';
                     $continueCheckingGroup = FALSE;
                  }
               }


               if ($continueCheckingGroup) if ($groupOwnerID === FALSE)
               {
                  echo '<br /><hr /><br />Could not determine group ownership of chpasswd.<br />Please verify that it is under group ownership of the group that your web server operates as.<br />';
                  $continueCheckingGroup = FALSE;
               }
               else
               {
                  $groupOwner = posix_getgrgid($groupOwnerID);
                  if (!$groupOwner)
                  {
                     echo '<br /><hr /><br />Could not determine group ownership of chpasswd.<br />Please verify that it is under group ownership of the group that your web server operates as.<br />';
                     $continueCheckingGroup = FALSE;
                  }
               }


               if ($continueCheckingGroup)
               {

                  echo '<br /><hr /><br />chpasswd has group ownership: ' 
                     . $groupOwner['name'] . '<br />';
                  echo 'Your web server is running under group: ' 
                     . $webGroup['name'] . '<br />';


                  // compare groups
                  //
                  if ($groupOwnerID != $webGroupID)
                     echo '<br />The chpasswd utility is not owned by the same group that your web server is!<br />Please read the INSTALL and README files for more information on how to set this correctly.<br />';

               }



               // ---------------------
               // check user ownership
               //
               if (!empty($pathToPwPolicy)) 
                  $userOwnerID = fileowner($pathToPwPolicy);
               else if (empty($overridePathToChpasswd))
                  $userOwnerID = fileowner(SM_PATH . 'plugins/change_passwd/chpasswd');
               else
                  $userOwnerID = fileowner($overridePathToChpasswd);


               $continueCheckingUser = TRUE;


               if ($userOwnerID === FALSE)
               {
                  echo '<br /><hr /><br />Could not determine ownership of chpasswd.<br />Please verify that it is owned by the root user.<br />';
               }
               else
               {

                  $userOwner = posix_getpwuid($userOwnerID);

                  if (!$userOwner)
                  {
                     echo '<br /><hr /><br />Could not determine ownership of chpasswd.<br />Please verify that it is owned by the root user.<br />';
                  }


                  else
                  {

                     echo '<br /><hr /><br />chpasswd is owned by: '
                        . $userOwner['name'] . '<br />';

                     if ($userOwnerID != 0)
                        echo '<br />The chpasswd utility is not owned by root!<br />Please read the INSTALL and README files for more information on how to set this correctly.<br />';

                  }

               }

            }


            // print info on how to test binary from command line
            //
            echo '<br /><hr /><br />To test the chpasswd utility from the command line, do this:<br /><br />' 
               . '<pre>cd ' . getcwd() . '</pre><pre>'
               . $cmd . '</pre><br /><hr />';
            exit;

         }
         //
         // end debug code
         // --------------------------------------------------------------------------------

         exec($cmd, $capt, $ret);

      }



      if (!$ret)
      {

         $msg = _("Your password was changed successfully");

         // write new cookies for the password
         //
         $onetimepad = OneTimePadCreate(strlen($new_pw1));
         sqsession_register($onetimepad, 'onetimepad');
         $key = OneTimePadEncrypt($new_pw1, $onetimepad);
         //setcookie('key', $key, 0, $base_uri);
         sqsetcookie('key', $key, 0, $base_uri);

      } 
      else
      { 

         $status = array(
                     0 => _("Your password was changed successfully"),
                     2 => _("Missing new password"),
                     3 => _("Missing current password"),
                     4 => sprintf(_("The password for this user cannot be changed due to security constraints: %s"), $user),
                     5 => _("The new password is equal to the current password. Choose another password."),
                     6 => _("Could not read password file"),  // not using shadow pwd file
                     7 => _("Could not read password file"),  // using shadow pwd file
                     8 => _("Temporary file could not be opened"),
                     9 => _("Your current password is not correct"),
                    10 => sprintf(_("User does not exist: %s"), $user),
                    11 => _("Virtual memory exhausted"),
                    12 => _("Missing username"),
                        );

         if (isset($status[$ret]))
            $msg = $status[$ret];
         else
            $msg = _("An error has occurred while attempting to change your password.") . ' '
                 . _("Please contact your system administrator.");

      }


      if ($seeOutput)
      {

         $msg .= '<br />' . _("Command output:") . ' ';

         for ($i = 0; $i < count($capt); $i++)
            $msg .= '<br />' . _($capt[$i]);

         $msg .= '<br />' . _("Return code:") . ' ' . $ret;

      }

   }

}


bindtextdomain('squirrelmail', SM_PATH . 'locale');
textdomain('squirrelmail');

displayPageHeader($color, 'None');

bindtextdomain('change_passwd', SM_PATH . 'locale');
textdomain('change_passwd');


?>
<br />
<table width="95%" align="center" border="0" cellpadding="2" cellspacing="0">
  <tr>
    <td align="center" bgcolor="<?php echo $color[3] ?>"><b><?php echo _("Change Password"); ?></b>
      <table cellspacing="0" cellpadding="0" border="0" width="99%">
        <tr>
          <td bgcolor="<?php echo $color[9] ?>">

            <?php 

               if (isset($msg))
                  echo "<table cellspacing=\"5\"><tr><td><b><font color=\"red\">$msg</font></b></td></tr></table>";
               echo '&nbsp;&nbsp;' 
                    . _("Please read the following instructions before editing your password:")
                    . '<ul>'
                    . '<li>' 
                    . _("You may use letters, numbers, and other special characters on your keyboard.")
                    . '<li>'
                    . _("Passwords are case-sensititve, so an 'A' is not the same as an 'a'.");

               if ($minimumPasswordLength > 0)
                  echo '<li>'
                       . sprintf(_("Your new password must be at least %s characters long."), 
                                 $minimumPasswordLength);

// add other criteria...
//               echo '<li>'
//                    . _("Your new password must contain at least three (3) letters (a-z) and two (2) digits (0-9).");

               echo '</ul>';

            ?>

            <form method="post">
              <input type="hidden" name="gochange" value="1" />
              <center>
                <hr style="height: 1px;" />
                <table>
                  <tr>
                    <td align="right"><?php echo _("Your user name:"); ?></td> 
                    <td><b><?php echo $username?></b>
                      <input type="hidden" name="user" value="<?php echo $username; ?>" />
                    </td>
                  </tr>
        <?php if ($confirmOldPass) { ?>
                  <tr>
                    <td align="right"><?php echo _("Current password:"); ?> </td>
                    <td><input name="old_pw" size="10" type="password" maxlength="50" /> </td>
                  </tr>
        <?php }?>
                  <tr>
                    <td align="right"><?php echo _("New password:"); ?> </td>
                    <td><input name="new_pw1" size="10" type="password" maxlength="50" /> </td>
                  </tr>
        <?php if ($confirmNewPass) { ?>
                  <tr>
                    <td align="right"><?php echo _("Confirm new password:"); ?> </td>
                    <td><input name="new_pw2" size="10" type="password" maxlength="50" /></td>
                  </tr>
        <?php }?>
                </table>
                <hr style="height: 1px;" />
                <input name="change" type="submit" value="<?php echo _("Change Password"); ?>" />
              </center>
            </form> 
          </td>
        </tr>
        <tr>
          <td>&nbsp;</td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</body>
</html>

<?php
   bindtextdomain('squirrelmail', SM_PATH . 'locale');
   textdomain('squirrelmail');
?>
